/******************************************************************************
 * Product: JPiere                                                            *
 * Copyright (C) Hideaki Hagiwara (h.hagiwara@oss-erp.co.jp)                  *
 *                                                                            *
 * This program is free software, you can redistribute it and/or modify it    *
 * under the terms version 2 of the GNU General Public License as published   *
 * by the Free Software Foundation. This program is distributed in the hope   *
 * that it will be useful, but WITHOUT ANY WARRANTY.                          *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * JPiere is maintained by OSS ERP Solutions Co., Ltd.                        *
 * (http://www.oss-erp.co.jp)                                                 *
 *****************************************************************************/
package jpiere.plugin.webui.sso.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.Properties;
import java.util.logging.Level;

import org.compiere.model.MClientInfo;
import org.compiere.util.CCache;
import org.compiere.util.CLogger;
import org.compiere.util.DB;
import org.compiere.util.Env;
import org.compiere.util.Msg;


/**
 *
 * JPIERE-0446
 *
 * @author h.hagiwara
 *
 */
public class MSAMLConfig extends X_JP_SAML_Config {

	/**	Cache						*/
	private static CCache<String,MSAMLConfig> s_cache = new CCache<String,MSAMLConfig>(Table_Name, 2);
	/**	Logger						*/
	private static CLogger		s_log = CLogger.getCLogger (MClientInfo.class);

	private static char[] usableCharacters = new char[]{
			'1', '2', '3', '4', '5', '6', '7', '8', '9', '0',
			'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
			'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'
		};

	public static final String JP_SAML_Parameter = "client";

	public MSAMLConfig(Properties ctx, int JP_SSO_Config_SAML_ID, String trxName)
	{
		super(ctx, JP_SSO_Config_SAML_ID, trxName);
	}

	public MSAMLConfig(Properties ctx, ResultSet rs, String trxName)
	{
		super(ctx, rs, trxName);
	}

	@Override
	protected boolean beforeSave(boolean newRecord)
	{

		if(newRecord || is_ValueChanged("JP_SAML_Parameter"))
		{
			if(JP_SAML_Parameter.equals(getJP_SAML_Parameter()))
			{
				;//Noting to do;
			}else {
				setJP_SAML_Parameter(JP_SAML_Parameter);
			}
		}

		if(newRecord || is_ValueChanged("JP_SAML_Parameter_Value"))
		{
			String characters = getJP_SAML_Parameter_Value();
			for(int i = 0; i < characters.length(); i++)
			{
				char target = characters.charAt(i);
				int index = Arrays.binarySearch(usableCharacters, target);
				if(index < 0)
				{
					Object[] objs = new Object[]{target};
					log.saveError("Error", Msg.getElement(getCtx(), "JP_SAML_Parameter_Value") + " : " + Msg.getMsg(Env.getCtx(),"JP_CanNotUseChar",objs));//You can not use this character : {0}.
					return false;
				}
			}
		}

		return true;
	}

	static public MSAMLConfig get(String parameter, String value)
	{
		String key = parameter + value;
		MSAMLConfig samlConfig = (MSAMLConfig)s_cache.get(key);
		if (samlConfig != null)
			return samlConfig;
		//
		String sql = "SELECT * FROM JP_SAML_Config WHERE JP_SAML_Parameter=? AND JP_SAML_Parameter_Value=?";
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try
		{
			pstmt = DB.prepareStatement (sql, null);
			pstmt.setString (1, parameter);
			pstmt.setString (2, value);
			rs = pstmt.executeQuery ();
			if (rs.next ())
			{
				samlConfig = new MSAMLConfig (Env.getCtx(), rs, null);
				s_cache.put (key, samlConfig);
			}
		}
		catch (SQLException ex)
		{
			s_log.log(Level.SEVERE, sql, ex);
		}
		finally
		{
			DB.close(rs, pstmt);
			rs = null;
			pstmt = null;
		}

		return samlConfig;

	}


}
