/******************************************************************************
 * Product: JPiere                                                            *
 * Copyright (C) Hideaki Hagiwara (h.hagiwara@oss-erp.co.jp)                  *
 *                                                                            *
 * This program is free software, you can redistribute it and/or modify it    *
 * under the terms version 2 of the GNU General Public License as published   *
 * by the Free Software Foundation. This program is distributed in the hope   *
 * that it will be useful, but WITHOUT ANY WARRANTY.                          *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * JPiere is maintained by OSS ERP Solutions Co., Ltd.                        *
 * (http://www.oss-erp.co.jp)                                                 *
 *****************************************************************************/
package jpiere.plugin.webui.sso.saml;

import java.security.KeyException;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.util.Base64;
import java.util.List;

import javax.xml.crypto.AlgorithmMethod;
import javax.xml.crypto.KeySelector;
import javax.xml.crypto.KeySelectorException;
import javax.xml.crypto.KeySelectorResult;
import javax.xml.crypto.XMLCryptoContext;
import javax.xml.crypto.XMLStructure;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.KeyValue;
import javax.xml.crypto.dsig.keyinfo.X509Data;

import org.compiere.util.Util;

import jpiere.plugin.webui.sso.model.MSAMLConfig;

/**
*
* JPIERE-0446
*
* @author h.hagiwara
*
*/
public class JPiereSAMLKeySelector extends KeySelector {

	private MSAMLConfig m_SAMLConfig = null;
	private String signScope = null;

	@Override
	public KeySelectorResult select(KeyInfo keyInfo, Purpose purpose, AlgorithmMethod method, XMLCryptoContext context) throws KeySelectorException
	{
		SignatureMethod sm = (SignatureMethod) method;

        if (keyInfo == null)
        {
    		String publicKeyString = null;
    		PublicKey pk = null;

    		if(m_SAMLConfig != null && signScope != null)
    		{
    			if(signScope.equals(LoginSAML.SIGN_SCOPE_Response))
    			{
    				publicKeyString = m_SAMLConfig.getJP_SAML_Response_PublicKey();

    			}else if(signScope.equals(LoginSAML.SIGN_SCOPE_ASSERTION)) {

    				publicKeyString = m_SAMLConfig.getJP_SAML_Assertion_PublicKey();

    			}
    		}

    		if(!Util.isEmpty(publicKeyString))
    		{
    			publicKeyString = publicKeyString.replaceAll("\\n", "").replace("-----BEGIN PUBLIC KEY-----", "").replace("-----END PUBLIC KEY-----", "");
                X509EncodedKeySpec keySpecX509 = new X509EncodedKeySpec(Base64.getDecoder().decode(publicKeyString));
                KeyFactory kf =null;
    			try {
    				kf = KeyFactory.getInstance("RSA");
    				RSAPublicKey pubKey = (RSAPublicKey) kf.generatePublic(keySpecX509);
    				pk = pubKey;
    			} catch (Exception e) {
    				throw new KeySelectorException(e);
    			}

    			if(pk != null)
    			{
                    if (algEquals(sm.getAlgorithm(), pk.getAlgorithm()))
                    {
                        return new JPiereSAMLKeySelectorResult(pk);
                    }else {
                    	throw new KeySelectorException("Null KeyInfo object!");
                    }

    			}else {
    				throw new KeySelectorException("Null KeyInfo object!");
    			}

    		}else {
    			throw new KeySelectorException("Null KeyInfo object!");
    		}


        }else {//if (keyInfo == null)


	        @SuppressWarnings("rawtypes")
			List list = keyInfo.getContent();

	        for (int i = 0; i < list.size(); i++)
	        {
	            XMLStructure xmlStructure = (XMLStructure) list.get(i);
	            if (xmlStructure instanceof KeyValue)
	            {
	        		String publicKeyString = null;
	        		PublicKey pk = null;

	        		if(m_SAMLConfig != null && signScope != null)
	        		{
	        			if(signScope.equals(LoginSAML.SIGN_SCOPE_Response))
	        			{
	        				publicKeyString = m_SAMLConfig.getJP_SAML_Response_PublicKey();

	        			}else if(signScope.equals(LoginSAML.SIGN_SCOPE_ASSERTION)) {

	        				publicKeyString = m_SAMLConfig.getJP_SAML_Assertion_PublicKey();

	        			}
	        		}

	        		if(!Util.isEmpty(publicKeyString))
	        		{
	        			publicKeyString = publicKeyString.replaceAll("\\n", "").replace("-----BEGIN PUBLIC KEY-----", "").replace("-----END PUBLIC KEY-----", "");
	                    X509EncodedKeySpec keySpecX509 = new X509EncodedKeySpec(Base64.getDecoder().decode(publicKeyString));
	                    KeyFactory kf =null;
	        			try {
	        				kf = KeyFactory.getInstance("RSA");
	        				RSAPublicKey pubKey = (RSAPublicKey) kf.generatePublic(keySpecX509);
	        				pk = pubKey;
	        			} catch (Exception e) {

	        				throw new KeySelectorException(e);
	        			}

	        			if(pk != null)
	        			{
	                        if (algEquals(sm.getAlgorithm(), pk.getAlgorithm()))
	                        {
	                            return new JPiereSAMLKeySelectorResult(pk);
	                        }
	        			}

	        		}else {

	        			try
	                    {
	                        pk = ((KeyValue)xmlStructure).getPublicKey();

	                    } catch (KeyException ke) {

	                        throw new KeySelectorException(ke);

	                    }

	                    // make sure algorithm is compatible with method
	                    if (algEquals(sm.getAlgorithm(), pk.getAlgorithm()))
	                    {
	                        return new JPiereSAMLKeySelectorResult(pk);
	                    }

	        		}

	            }else if(xmlStructure instanceof X509Data) {

	//            	 for (Object x509Content : ((X509Data)xmlStructure).getContent()) {
	//                     X509Certificate cert = (X509Certificate)x509Content;
	//                     return new JPiereSAMLKeySelectorResult(cert.getPublicKey());
	//                 }

	            }

	        }//for

        }//if (keyInfo == null)

        throw new KeySelectorException("No KeyValue element found!");
	}

    static boolean algEquals(String algURI, String algName)
    {
    	if (algName.equalsIgnoreCase("DSA") && algURI.equalsIgnoreCase(SignatureMethod.DSA_SHA1))
    	{
    		return true;
    	}else if (algName.equalsIgnoreCase("RSA") && algURI.equalsIgnoreCase(SignatureMethod.RSA_SHA1)) {
            return true;
    	}else if (algName.equalsIgnoreCase("RSA") && algURI.equalsIgnoreCase(SignatureMethod.RSA_SHA256)) {
            return true;
    	}else if (algName.equalsIgnoreCase("RSA") && algURI.equalsIgnoreCase(SignatureMethod.RSA_SHA512)) {
            return true;
    	}else {
            return false;
    	}
    }

    public void setMSAMLConfig(MSAMLConfig m_SAMLConfig)
    {
    	this.m_SAMLConfig = m_SAMLConfig;
    }

    public void setSignScope(String ss)
    {
    	this.signScope = ss;
    }
}
