/******************************************************************************
 * Product: JPiere                                                            *
 * Copyright (C) Hideaki Hagiwara (h.hagiwara@oss-erp.co.jp)                  *
 *                                                                            *
 * This program is free software, you can redistribute it and/or modify it    *
 * under the terms version 2 of the GNU General Public License as published   *
 * by the Free Software Foundation. This program is distributed in the hope   *
 * that it will be useful, but WITHOUT ANY WARRANTY.                          *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * JPiere is maintained by OSS ERP Solutions Co., Ltd.                        *
 * (http://www.oss-erp.co.jp)                                                 *
 *****************************************************************************/
package jpiere.plugin.webui.sso.saml;

import java.io.InputStream;
import java.security.KeyStore;
import java.util.HashMap;
import java.util.Map;

import org.opensaml.core.criterion.EntityIdCriterion;
import org.opensaml.security.credential.Credential;
import org.opensaml.security.credential.impl.KeyStoreCredentialResolver;

import net.shibboleth.utilities.java.support.resolver.CriteriaSet;
import net.shibboleth.utilities.java.support.resolver.Criterion;
import net.shibboleth.utilities.java.support.resolver.ResolverException;

/**
*
* JPIERE-0446
*
* @author h.hagiwara
*
*/
public class JPiereSPCredentials {

	  private static final String KEY_STORE_PASSWORD = "password";
	    private static final String KEY_STORE_ENTRY_PASSWORD = "password";
	    private static final String KEY_STORE_PATH = "/SPKeystore.jks";
	    private static final String KEY_ENTRY_ID = "SPKey";

	    private static final Credential credential;

	    static {
	        try {
	            KeyStore keystore = readKeystoreFromFile(KEY_STORE_PATH, KEY_STORE_PASSWORD);
	            Map<String, String> passwordMap = new HashMap<String, String>();
	            passwordMap.put(KEY_ENTRY_ID, KEY_STORE_ENTRY_PASSWORD);
	            KeyStoreCredentialResolver resolver = new KeyStoreCredentialResolver(keystore, passwordMap);

	            Criterion criterion = new EntityIdCriterion(KEY_ENTRY_ID);
	            CriteriaSet criteriaSet = new CriteriaSet();
	            criteriaSet.add(criterion);

	            credential = resolver.resolveSingle(criteriaSet);

	        } catch (ResolverException e) {
	            throw new RuntimeException("Something went wrong reading credentials", e);
	        }
	    }

	    private static KeyStore readKeystoreFromFile(String pathToKeyStore, String keyStorePassword) {
	        try {
	            KeyStore keystore = KeyStore.getInstance(KeyStore.getDefaultType());
	            InputStream inputStream = JPiereSPCredentials.class.getResourceAsStream(pathToKeyStore);
	            keystore.load(inputStream, keyStorePassword.toCharArray());
	            inputStream.close();
	            return keystore;
	        } catch (Exception e) {
	            throw new RuntimeException("Something went wrong reading keystore", e);
	        }
	    }

	    public static Credential getCredential() {
	        return credential;
	    }
}
