/******************************************************************************
 * Product: Posterita Ajax UI 												  *
 * Copyright (C) 2007 Posterita Ltd.  All Rights Reserved.                    *
 * This program is free software; you can redistribute it and/or modify it    *
 * under the terms version 2 of the GNU General Public License as published   *
 * by the Free Software Foundation. This program is distributed in the hope   *
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.           *
 * See the GNU General Public License for more details.                       *
 * You should have received a copy of the GNU General Public License along    *
 * with this program; if not, write to the Free Software Foundation, Inc.,    *
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.                     *
 * For the text or an alternative of this public license, you may reach us    *
 * Posterita Ltd., 3, Draper Avenue, Quatre Bornes, Mauritius                 *
 * or via info@posterita.org or http://www.posterita.org/                     *
 *****************************************************************************/

package jpiere.plugin.webui.sso.saml;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.lang.ref.WeakReference;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Base64;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.adempiere.util.LogAuthFailure;
import org.adempiere.util.ServerContext;
import org.adempiere.webui.ClientInfo;
import org.adempiere.webui.IWebClient;
import org.adempiere.webui.WLogin;
import org.adempiere.webui.apps.AEnv;
import org.adempiere.webui.component.DrillCommand;
import org.adempiere.webui.component.TokenCommand;
import org.adempiere.webui.component.ZoomCommand;
import org.adempiere.webui.desktop.DefaultDesktop;
import org.adempiere.webui.desktop.FavouriteController;
import org.adempiere.webui.desktop.IDesktop;
import org.adempiere.webui.session.SessionContextListener;
import org.adempiere.webui.session.SessionManager;
import org.adempiere.webui.theme.ThemeManager;
import org.adempiere.webui.util.BrowserToken;
import org.adempiere.webui.util.UserPreference;
import org.adempiere.webui.window.FDialog;
import org.compiere.model.MAcctSchema;
import org.compiere.model.MClient;
import org.compiere.model.MClientInfo;
import org.compiere.model.MCountry;
import org.compiere.model.MOrg;
import org.compiere.model.MRole;
import org.compiere.model.MSession;
import org.compiere.model.MSysConfig;
import org.compiere.model.MSystem;
import org.compiere.model.MTable;
import org.compiere.model.MUser;
import org.compiere.model.MUserPreference;
import org.compiere.model.MUserRoles;
import org.compiere.model.MWarehouse;
import org.compiere.model.ModelValidationEngine;
import org.compiere.util.CLogger;
import org.compiere.util.DB;
import org.compiere.util.Env;
import org.compiere.util.Language;
import org.compiere.util.Msg;
import org.compiere.util.Util;
import org.joda.time.DateTime;
import org.opensaml.core.config.InitializationException;
import org.opensaml.core.config.InitializationService;
import org.opensaml.messaging.context.MessageContext;
import org.opensaml.messaging.encoder.MessageEncodingException;
import org.opensaml.saml.common.messaging.context.SAMLEndpointContext;
import org.opensaml.saml.common.messaging.context.SAMLPeerEntityContext;
import org.opensaml.saml.common.xml.SAMLConstants;
import org.opensaml.saml.saml2.binding.encoding.impl.HTTPRedirectDeflateEncoder;
import org.opensaml.saml.saml2.core.AuthnContext;
import org.opensaml.saml.saml2.core.AuthnContextClassRef;
import org.opensaml.saml.saml2.core.AuthnContextComparisonTypeEnumeration;
import org.opensaml.saml.saml2.core.AuthnRequest;
import org.opensaml.saml.saml2.core.Issuer;
import org.opensaml.saml.saml2.core.NameIDPolicy;
import org.opensaml.saml.saml2.core.NameIDType;
import org.opensaml.saml.saml2.core.RequestedAuthnContext;
import org.opensaml.saml.saml2.metadata.SingleSignOnService;
import org.opensaml.xmlsec.SignatureSigningParameters;
import org.opensaml.xmlsec.config.JavaCryptoValidationInitializer;
import org.opensaml.xmlsec.context.SecurityParametersContext;
import org.opensaml.xmlsec.signature.support.SignatureConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.zkforge.keylistener.Keylistener;
import org.zkoss.web.Attributes;
import org.zkoss.web.servlet.Servlets;
import org.zkoss.zk.au.out.AuScript;
import org.zkoss.zk.ui.Desktop;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.Session;
import org.zkoss.zk.ui.event.ClientInfoEvent;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.Events;
import org.zkoss.zk.ui.sys.DesktopCache;
import org.zkoss.zk.ui.sys.SessionCtrl;
import org.zkoss.zk.ui.util.Clients;
import org.zkoss.zul.Window;

import jpiere.plugin.webui.sso.model.MSAMLConfig;
import net.shibboleth.utilities.java.support.component.ComponentInitializationException;

/**
 *
 * @author  <a href="mailto:agramdass@gmail.com">Ashley G Ramdass</a>
 * @date    Feb 25, 2007
 * @version $Revision: 0.10 $
 *
 * @author hengsin
 *
 * @author Hideaki Hagiwara
 *
 */
public class LoginSAML extends Window implements EventListener<Event>, IWebClient
{
	/**
	 *
	 */
	private static final long serialVersionUID = -3320656546509525766L;

	private static final String SAVED_CONTEXT = "saved.context";

	public static final String APPLICATION_DESKTOP_KEY = "application.desktop";

	public static String APP_NAME = null;

    public static final String UID          = "1.0.0";

    public static final String WIDGET_INSTANCE_NAME = "instanceName";

    private WLogin             loginDesktop;

    private ClientInfo		   clientInfo = new ClientInfo();

	private String langSession;

	private UserPreference userPreference;

	private MUserPreference userPreferences;

	private Keylistener keyListener;

	private static final CLogger logger = CLogger.getCLogger(LoginSAML.class);

	public static final String EXECUTION_CARRYOVER_SESSION_KEY = "execution.carryover";

	private static final String CLIENT_INFO = "client.info";

	private static boolean eventThreadEnabled = false;

	private ConcurrentMap<String, String[]> m_URLParameters;

	private static LogAuthFailure logAuthFailure = new LogAuthFailure();

    public LoginSAML()
    {
    	this.setVisible(false);

    	userPreference = new UserPreference();
    	// preserve the original URL parameters as is destroyed later on loging
    	m_URLParameters = new ConcurrentHashMap<String, String[]>(Executions.getCurrent().getParameterMap());
    }

	public void onCreate()
    {
        this.getPage().setTitle(ThemeManager.getBrowserTitle());

        SessionManager.setSessionApplication(this);
        Session session = Executions.getCurrent().getDesktop().getSession();

        /** SSO **/
        HttpServletRequest httpServletRequest = (HttpServletRequest)Executions.getCurrent().getNativeRequest();
        HttpServletResponse httpServletResponse =  (HttpServletResponse)Executions.getCurrent().getNativeResponse();
        Properties ctx = Env.getCtx();

    	String paramValue = httpServletRequest.getParameter(MSAMLConfig.JP_SAML_Parameter) ;
		MSAMLConfig m_SAMLConfig =  MSAMLConfig.get(MSAMLConfig.JP_SAML_Parameter, paramValue);
		if(m_SAMLConfig == null)
		{
			try {
				httpServletResponse.sendRedirect("index.zul");
			} catch (IOException e) {
				e.printStackTrace();
			}
			return ;
		}

        //SAML
       	if (httpServletRequest.getParameter("SAMLResponse") == null)
        {
    		redirectSAML(ctx, session, httpServletRequest, httpServletResponse, m_SAMLConfig);
    		return ;

        }else {

        	try
        	{
        		loginProcedureBySAML(ctx, session, httpServletRequest, httpServletResponse, m_SAMLConfig);

        	}catch (Exception e) {

                langSession = Env.getContext(ctx, Env.LANGUAGE);
                if (session.getAttribute(SessionContextListener.SESSION_CTX) == null || !SessionManager.isUserLoggedIn(ctx))
                {
                    loginDesktop = new WLogin(this);
                    loginDesktop.createPart(this.getPage());
                    loginDesktop.getComponent().getRoot().addEventListener(Events.ON_CLIENT_INFO, this);

                }
                else
                {
                    loginCompleted();
                }

                String x_Forward_IP = Executions.getCurrent().getHeader("X-Forwarded-For");
                if (x_Forward_IP == null)
                	 x_Forward_IP = Executions.getCurrent().getRemoteAddr();

                //The SSO login attempt failed.
                if(m_SAMLConfig.isVerboseJP())
                {
                	StringWriter sw = new StringWriter();
                	PrintWriter pw = new PrintWriter(sw);
                	e.printStackTrace(pw);
                	String stackTrace = sw.toString();

               		logAuthFailure.log(x_Forward_IP, "/webui/" + m_SAMLConfig.getJP_SAML_ServletFile() + "?" + MSAMLConfig.JP_SAML_Parameter + "=" + paramValue
								, "SAML", stackTrace);

                	FDialog.error(0, "JP_SSO_FAILED", e.getMessage());

                }else {

                	logAuthFailure.log(x_Forward_IP, "/webui/" + m_SAMLConfig.getJP_SAML_ServletFile() + "?" + MSAMLConfig.JP_SAML_Parameter + "=" + paramValue
							, "SAML", e.getMessage());

                	FDialog.error(0, "JP_SSO_FAILED");

                }

                Executions.getCurrent().getDesktop().enableServerPush(true);

        		Executions.getCurrent().getDesktop().addListener(new DrillCommand());
   		        Executions.getCurrent().getDesktop().addListener(new TokenCommand());
   		        Executions.getCurrent().getDesktop().addListener(new ZoomCommand());

   		        eventThreadEnabled = Executions.getCurrent().getDesktop().getWebApp().getConfiguration().isEventThreadEnabled();

			}

        	return ;
        }

   }//onCreate()

    public void onOk()
    {
    }

    public void onCancel()
    {
    }

    /* (non-Javadoc)
	 * @see org.adempiere.webui.IWebClient#loginCompleted()
	 */
    public void loginCompleted()
    {
    	if (loginDesktop != null)
    	{
    		loginDesktop.getComponent().getRoot().removeEventListener(Events.ON_CLIENT_INFO, this);
    		loginDesktop.detach();
    		loginDesktop = null;
    	}

        Properties ctx = Env.getCtx();
        String langLogin = Env.getContext(ctx, Env.LANGUAGE);
        if (langLogin == null || langLogin.length() <= 0)
        {
        	langLogin = langSession;
        	Env.setContext(ctx, Env.LANGUAGE, langSession);
        }

        MSystem system = MSystem.get(Env.getCtx());
        Env.setContext(ctx, "#System_Name", system.getName());

        // Validate language
		Language language = Language.getLanguage(langLogin);
		String locale = Env.getContext(ctx, AEnv.LOCALE);
		if (locale != null && locale.length() > 0 && !language.getLocale().toString().equals(locale))
		{
			String adLanguage = language.getAD_Language();
			Language tmp = Language.getLanguage(locale);
			language = new Language(tmp.getName(), adLanguage, tmp.getLocale(), tmp.isDecimalPoint(),
	    			tmp.getDateFormat().toPattern(), tmp.getMediaSize());
		}
		else
		{
			Language tmp = language;
			language = new Language(tmp.getName(), tmp.getAD_Language(), tmp.getLocale(), tmp.isDecimalPoint(),
	    			tmp.getDateFormat().toPattern(), tmp.getMediaSize());
		}
    	Env.verifyLanguage(ctx, language);
    	Env.setContext(ctx, Env.LANGUAGE, language.getAD_Language()); //Bug

    	StringBuilder calendarMsgScript = new StringBuilder();
		String monthMore = Msg.getMsg(ctx,"more");
		String dayMore = Msg.getMsg(ctx,"more");
		calendarMsgScript.append("function _overrideMsgCal() { msgcal.monthMORE = '+{0} ")
			.append(monthMore).append("';");
		calendarMsgScript.append("msgcal.dayMORE = '+{0} ")
			.append(dayMore).append("'; }");
		AuScript auscript = new AuScript(calendarMsgScript.toString());
		Clients.response(auscript);

		//	Create adempiere Session - user id in ctx
        Session currSess = Executions.getCurrent().getDesktop().getSession();
        HttpSession httpSess = (HttpSession) currSess.getNativeSession();
        String x_Forward_IP = Executions.getCurrent().getHeader("X-Forwarded-For");

		MSession mSession = MSession.get (ctx, x_Forward_IP!=null ? x_Forward_IP : Executions.getCurrent().getRemoteAddr(),
			Executions.getCurrent().getRemoteHost(), httpSess.getId() );
		if (clientInfo.userAgent != null) {
			mSession.setDescription(mSession.getDescription() + "\n" + clientInfo.toString());
			mSession.saveEx();
		}

		currSess.setAttribute("Check_AD_User_ID", Env.getAD_User_ID(ctx));

		//enable full interface, relook into this when doing preference
		Env.setContext(ctx, "#ShowTrl", true);
		Env.setContext(ctx, "#ShowAcct", MRole.getDefault().isShowAcct());

		// to reload preferences when the user refresh the browser
		userPreference = loadUserPreference(Env.getAD_User_ID(ctx));
    	userPreferences = MUserPreference.getUserPreference(Env.getAD_User_ID(ctx), Env.getAD_Client_ID(ctx));

		//auto commit user preference
    	userPreferences.fillPreferences();

		keyListener = new Keylistener();
		keyListener.setPage(this.getPage());
		keyListener.setCtrlKeys("@a@c@d@e@f@h@n@o@p@r@s@t@z@x@#left@#right@#up@#down@#home@#end#enter^u@u@#pgdn@#pgup");
		keyListener.setAutoBlur(false);

		//create new desktop
		IDesktop appDesktop = createDesktop();
		appDesktop.setClientInfo(clientInfo);
		appDesktop.createPart(this.getPage());
		this.getPage().getDesktop().setAttribute(APPLICATION_DESKTOP_KEY, new WeakReference<IDesktop>(appDesktop));
		appDesktop.getComponent().getRoot().addEventListener(Events.ON_CLIENT_INFO, this);

		//track browser tab per session
		SessionContextListener.addDesktopId(mSession.getAD_Session_ID(), getPage().getDesktop().getId());

		//ensure server push is on
		if (!this.getPage().getDesktop().isServerPushEnabled())
			this.getPage().getDesktop().enableServerPush(true);

		//update session context
		currSess.setAttribute(SessionContextListener.SESSION_CTX, ServerContext.getCurrentInstance());

		MUser user = MUser.get(ctx);
		BrowserToken.save(mSession, user);

		Env.setContext(ctx, "#UIClient", "zk");
		Env.setContext(ctx, "#DBType", DB.getDatabase().getName());
		StringBuilder localHttpAddr = new StringBuilder(Executions.getCurrent().getScheme());
		localHttpAddr.append("://").append(Executions.getCurrent().getLocalAddr());
		int port = Executions.getCurrent().getLocalPort();
		if (port > 0 && port != 80) {
			localHttpAddr.append(":").append(port);
		}
		Env.setContext(ctx, "#LocalHttpAddr", localHttpAddr.toString());
		Clients.response(new AuScript("zAu.cmd0.clearBusy()"));

		//init favorite
		FavouriteController.getInstance(currSess);

		processParameters();
    }

    private void processParameters() {
    	String action = getPrmString("Action");
    	if ("Zoom".equalsIgnoreCase(action)) {
    		int tableID = getPrmInt("AD_Table_ID");
    		if (tableID == 0) {
    			String tableName = getPrmString("TableName");
    			if (!Util.isEmpty(tableName)) {
    				MTable table = MTable.get(Env.getCtx(), tableName);
    				if (table != null) {
    					tableID = table.getAD_Table_ID();
    				}
    			}
    		}
    		int recordID = getPrmInt("Record_ID");
    		if (tableID > 0) {
    			AEnv.zoom(tableID, recordID);
    		}
    	}
    	m_URLParameters = null;
    }

    private String getPrmString(String prm) {
    	String retValue = "";
    	if (m_URLParameters != null) {
        	String[] strs = m_URLParameters.get(prm);
        	if (strs != null && strs.length == 1 && strs[0] != null)
        		retValue = strs[0];
    	}
    	return retValue;
    }

    private int getPrmInt(String prm) {
    	int retValue = 0;
    	String str = getPrmString(prm);
		try {
    		if (!Util.isEmpty(str))
    			retValue = Integer.parseInt(str);
		} catch (NumberFormatException e) {
			// ignore
		}
    	return retValue;
    }

	/**
     * @return key listener
     */
    @Override
	public Keylistener getKeylistener() {
    	return keyListener;
    }

    private IDesktop createDesktop()
    {
    	IDesktop appDesktop = null;
		String className = MSysConfig.getValue(MSysConfig.ZK_DESKTOP_CLASS);
		if ( className != null && className.trim().length() > 0)
		{
			try
			{
				Class<?> clazz = this.getClass().getClassLoader().loadClass(className);
				appDesktop = (IDesktop) clazz.getDeclaredConstructor().newInstance();
			}
			catch (Throwable t)
			{
				logger.warning("Failed to instantiate desktop. Class=" + className);
			}
		}
		//fallback to default
		if (appDesktop == null)
			appDesktop = new DefaultDesktop();

		return appDesktop;
	}

	/* (non-Javadoc)
	 * @see org.adempiere.webui.IWebClient#logout()
	 */
    public void logout()
    {
    	Session session = logout0();
    	DesktopCache desktopCache = ((SessionCtrl)session).getDesktopCache();

    	//clear context, invalidate session
    	Env.getCtx().clear();
    	session.invalidate();

        //redirect to login page
        Executions.sendRedirect("index.zul");

        if (desktopCache != null)
			desktopCache.removeDesktop(Executions.getCurrent().getDesktop());
    }
    public void logoutAfterTabDestroyed(){
       	Session session = logout0();

    	//clear context, invalidate session
    	Env.getCtx().clear();
    	session.invalidate();
    }


	protected Session logout0() {
		Session session = Executions.getCurrent().getDesktop().getSession();

		if (keyListener != null) {
			keyListener.detach();
			keyListener = null;
		}

		//stop background thread
		IDesktop appDesktop = getAppDeskop();
		if (appDesktop != null)
			appDesktop.logout();

    	//clear remove all children and root component
    	getChildren().clear();
    	getPage().removeComponents();

    	//clear session attributes
		session.getAttributes().clear();

    	//logout ad_session
    	AEnv.logout();
		return session;
	}

    /**
     * @return IDesktop
     */
    public IDesktop getAppDeskop()
    {
    	Desktop desktop = Executions.getCurrent() != null ? Executions.getCurrent().getDesktop() : null;
    	IDesktop appDesktop = null;
    	if (desktop != null)
    	{
    		@SuppressWarnings("unchecked")
			WeakReference<IDesktop> ref = (WeakReference<IDesktop>) desktop.getAttribute(APPLICATION_DESKTOP_KEY);
    		if (ref != null)
    		{
    			appDesktop = ref.get();
    		}
    	}

    	return appDesktop;
    }

	public void onEvent(Event event) {
		if (event instanceof ClientInfoEvent) {
			ClientInfoEvent c = (ClientInfoEvent)event;
			clientInfo = new ClientInfo();
			clientInfo.colorDepth = c.getColorDepth();
			clientInfo.screenHeight = c.getScreenHeight();
			clientInfo.screenWidth = c.getScreenWidth();
			clientInfo.devicePixelRatio = c.getDevicePixelRatio();
			clientInfo.desktopHeight = c.getDesktopHeight();
			clientInfo.desktopWidth = c.getDesktopWidth();
			clientInfo.desktopXOffset = c.getDesktopXOffset();
			clientInfo.desktopYOffset = c.getDesktopYOffset();
			clientInfo.orientation = c.getOrientation();
			clientInfo.timeZone = c.getTimeZone();
			String ua = Servlets.getUserAgent((ServletRequest) Executions.getCurrent().getNativeRequest());
			clientInfo.userAgent = ua;
			ua = ua.toLowerCase();
			clientInfo.tablet = false;
			if (Executions.getCurrent().getBrowser("mobile") !=null) {
				clientInfo.tablet = true;
			} else if (ua.contains("ipad") || ua.contains("iphone") || ua.contains("android")) {
				clientInfo.tablet = true;
			}
			if (getDesktop() != null && getDesktop().getSession() != null) {
				getDesktop().getSession().setAttribute(CLIENT_INFO, clientInfo);
			} else if (Executions.getCurrent() != null){
				Executions.getCurrent().getSession().setAttribute(CLIENT_INFO, clientInfo);
			}

			Env.setContext(Env.getCtx(), "#clientInfo_desktopWidth", clientInfo.desktopWidth);
			Env.setContext(Env.getCtx(), "#clientInfo_desktopHeight", clientInfo.desktopHeight);
			Env.setContext(Env.getCtx(), "#clientInfo_orientation", clientInfo.orientation);
			Env.setContext(Env.getCtx(), "#clientInfo_mobile", clientInfo.tablet);

			IDesktop appDesktop = getAppDeskop();
			if (appDesktop != null)
				appDesktop.setClientInfo(clientInfo);

		}

	}

	private void onChangeRole(Map<String, Object> map) {
		Locale locale = (Locale) map.get("locale");
		Properties properties = (Properties) map.get("context");

		SessionManager.setSessionApplication(this);
		loginDesktop = new WLogin(this);
        loginDesktop.createPart(this.getPage());
        loginDesktop.changeRole(locale, properties);
        loginDesktop.getComponent().getRoot().addEventListener(Events.ON_CLIENT_INFO, this);
	}

	/**
	 * @param userId
	 * @return UserPreference
	 */
	public UserPreference loadUserPreference(int userId) {
		userPreference.loadPreference(userId);
		return userPreference;
	}

	/**
	 * @return UserPrerence
	 */
	public UserPreference getUserPreference() {
		return userPreference;
	}

	public static boolean isEventThreadEnabled() {
		return eventThreadEnabled;
	}

	@Override
	public void changeRole(MUser user) {
		//save context for re-login
		Properties properties = new Properties();
		Env.setContext(properties, Env.AD_CLIENT_ID, Env.getAD_Client_ID(Env.getCtx()));
		Env.setContext(properties, Env.AD_ORG_ID, Env.getAD_Org_ID(Env.getCtx()));
		Env.setContext(properties, Env.AD_USER_ID, user.getAD_User_ID());
		Env.setContext(properties, Env.AD_ROLE_ID, Env.getAD_Role_ID(Env.getCtx()));
		Env.setContext(properties, Env.AD_ORG_NAME, Env.getContext(Env.getCtx(), Env.AD_ORG_NAME));
		Env.setContext(properties, Env.M_WAREHOUSE_ID, Env.getContext(Env.getCtx(), Env.M_WAREHOUSE_ID));
		Env.setContext(properties, UserPreference.LANGUAGE_NAME, Env.getContext(Env.getCtx(), UserPreference.LANGUAGE_NAME));
		Env.setContext(properties, Env.LANGUAGE, Env.getContext(Env.getCtx(), Env.LANGUAGE));
		Env.setContext(properties, AEnv.LOCALE, Env.getContext(Env.getCtx(), AEnv.LOCALE));

		Locale locale = (Locale) Executions.getCurrent().getDesktop().getSession().getAttribute(Attributes.PREFERRED_LOCALE);
		HttpServletRequest httpRequest = (HttpServletRequest) Executions.getCurrent().getNativeRequest();

		Session session = logout0();

    	//clear context and invalidate session
		Env.getCtx().clear();
    	((SessionCtrl)session).invalidateNow();

    	//put saved context into new session
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("context", properties);
		map.put("locale", locale);

		HttpSession newSession = httpRequest.getSession(true);
		newSession.setAttribute(SAVED_CONTEXT, map);
		properties.setProperty(SessionContextListener.SERVLET_SESSION_ID, newSession.getId());

		Executions.sendRedirect("index.zul");
	}

	@Override
	public ClientInfo getClientInfo() {
		return clientInfo;
	}

	/**
	 * @return string for setupload
	 */
	public static String getUploadSetting() {
		StringBuilder uploadSetting = new StringBuilder("true,native");
		int size = MSysConfig.getIntValue(MSysConfig.ZK_MAX_UPLOAD_SIZE, 0);
		if (size > 0) {
			uploadSetting.append(",maxsize=").append(size);
		}
		return uploadSetting.toString();
	}


	/**
	 *
	 * @param session
	 * @param httpServletRequest
	 * @param httpServletResponse
	 */
	private void redirectSAML(Properties ctx, Session session,
			HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, MSAMLConfig m_SAMLConfig )
	{

        JavaCryptoValidationInitializer javaCryptoValidationInitializer = new JavaCryptoValidationInitializer();
        try {
            javaCryptoValidationInitializer.init();
        } catch (InitializationException e) {
            e.printStackTrace();
        }

        try {
            InitializationService.initialize();
        } catch (InitializationException e) {
            throw new RuntimeException("Initialization failed");
        }

        AuthnRequest authnRequest = JPiereOpenSAMLUtils.buildSAMLObject(AuthnRequest.class);
        authnRequest.setIssueInstant(new DateTime());
        authnRequest.setDestination(m_SAMLConfig.getJP_SAML_Redirect_URL());//Redirect URI of IdP

        if(m_SAMLConfig.getJP_SAML_Binding() == null)
        	authnRequest.setProtocolBinding(SAMLConstants.SAML2_POST_BINDING_URI);
        else if(m_SAMLConfig.getJP_SAML_Binding().equals(MSAMLConfig.JP_SAML_BINDING_PostBinding))
        	authnRequest.setProtocolBinding(SAMLConstants.SAML2_POST_BINDING_URI);
        else if(m_SAMLConfig.getJP_SAML_Binding().equals(MSAMLConfig.JP_SAML_BINDING_RedirectBinding))
            authnRequest.setProtocolBinding(SAMLConstants.SAML2_REDIRECT_BINDING_URI);
        else if(m_SAMLConfig.getJP_SAML_Binding().equals(MSAMLConfig.JP_SAML_BINDING_ArtifactBinding))
        	authnRequest.setProtocolBinding(SAMLConstants.SAML2_ARTIFACT_BINDING_URI);
        else
        	authnRequest.setProtocolBinding(SAMLConstants.SAML2_POST_BINDING_URI);

        authnRequest.setAssertionConsumerServiceURL(httpServletRequest.getRequestURL().toString()+"?"+m_SAMLConfig.getJP_SAML_Parameter()+"="+m_SAMLConfig.getJP_SAML_Parameter_Value());//Redirect URI of SP
        authnRequest.setID(JPiereOpenSAMLUtils.generateSecureRandomId());

        //Build Issuer
        if(m_SAMLConfig.getJP_SAML_Request_Issuer() != null)
        {
	        Issuer issuer = (Issuer)JPiereOpenSAMLUtils.buildSAMLObject(Issuer.class);
	        issuer.setValue(m_SAMLConfig.getJP_SAML_Request_Issuer()); //Login Requester = Clients of KEYCLOAK = ID of SP
	        authnRequest.setIssuer(issuer);
        }

        //Build NameID Policy
        NameIDPolicy nameIDPolicy = JPiereOpenSAMLUtils.buildSAMLObject(NameIDPolicy.class);
        nameIDPolicy.setAllowCreate(true);

        if(m_SAMLConfig.getJP_SAML_NameIDPolicy() == null)
        	nameIDPolicy.setFormat(NameIDType.UNSPECIFIED);
        else if(m_SAMLConfig.getJP_SAML_NameIDPolicy().equals(MSAMLConfig.JP_SAML_NAMEIDPOLICY_Unspecified))
        	nameIDPolicy.setFormat(NameIDType.UNSPECIFIED);
        else if(m_SAMLConfig.getJP_SAML_NameIDPolicy().equals(MSAMLConfig.JP_SAML_NAMEIDPOLICY_E_MailAddress))
        	nameIDPolicy.setFormat(NameIDType.EMAIL);
        else if(m_SAMLConfig.getJP_SAML_NameIDPolicy().equals(MSAMLConfig.JP_SAML_NAMEIDPOLICY_Entity))
        	nameIDPolicy.setFormat(NameIDType.ENTITY);
        else if(m_SAMLConfig.getJP_SAML_NameIDPolicy().equals(MSAMLConfig.JP_SAML_NAMEIDPOLICY_Kerberos))
        	nameIDPolicy.setFormat(NameIDType.KERBEROS);
        else if(m_SAMLConfig.getJP_SAML_NameIDPolicy().equals(MSAMLConfig.JP_SAML_NAMEIDPOLICY_Persistent))
        	nameIDPolicy.setFormat(NameIDType.PERSISTENT);
        else if(m_SAMLConfig.getJP_SAML_NameIDPolicy().equals(MSAMLConfig.JP_SAML_NAMEIDPOLICY_Transient))
        	nameIDPolicy.setFormat(NameIDType.TRANSIENT);
        else
        	nameIDPolicy.setFormat(NameIDType.UNSPECIFIED);

        authnRequest.setNameIDPolicy(nameIDPolicy);

        //Build Reuested Authn Context
        RequestedAuthnContext requestedAuthnContext = JPiereOpenSAMLUtils.buildSAMLObject(RequestedAuthnContext.class);
        requestedAuthnContext.setComparison(AuthnContextComparisonTypeEnumeration.MINIMUM);

        AuthnContextClassRef passwordAuthnContextClassRef = JPiereOpenSAMLUtils.buildSAMLObject(AuthnContextClassRef.class);
        passwordAuthnContextClassRef.setAuthnContextClassRef(AuthnContext.PASSWORD_AUTHN_CTX);
        requestedAuthnContext.getAuthnContextClassRefs().add(passwordAuthnContextClassRef);
        authnRequest.setRequestedAuthnContext(requestedAuthnContext);

    	MessageContext context = new MessageContext();
        context.setMessage(authnRequest);

        SAMLPeerEntityContext peerEntityContext = context.getSubcontext(SAMLPeerEntityContext.class, true);
        SAMLEndpointContext endpointContext = peerEntityContext.getSubcontext(SAMLEndpointContext.class, true);

        //
        SingleSignOnService endpoint = JPiereOpenSAMLUtils.buildSAMLObject(SingleSignOnService.class);
        endpoint.setBinding(SAMLConstants.SAML2_REDIRECT_BINDING_URI);
        endpoint.setLocation(m_SAMLConfig.getJP_SAML_Redirect_URL());
        endpointContext.setEndpoint(endpoint);

        boolean isSign = false;

        if(isSign)
        {
	        SignatureSigningParameters signatureSigningParameters = new SignatureSigningParameters();
	        signatureSigningParameters.setSigningCredential(JPiereSPCredentials.getCredential());
	        signatureSigningParameters.setSignatureAlgorithm(SignatureConstants.ALGO_ID_SIGNATURE_RSA_SHA256);
	        context.getSubcontext(SecurityParametersContext.class, true).setSignatureSigningParameters(signatureSigningParameters);
        }

        HTTPRedirectDeflateEncoder encoder = new HTTPRedirectDeflateEncoder();

        encoder.setMessageContext(context);
        encoder.setHttpServletResponse(httpServletResponse);

        try {
            encoder.initialize();
        } catch (ComponentInitializationException e) {
            throw new RuntimeException(e);
        }

        JPiereOpenSAMLUtils.logSAMLObject(authnRequest);

        try {
            encoder.encode();
        } catch (MessageEncodingException e) {
            throw new RuntimeException(e);
        }

        return ;

	}

	private static String NS_PROTOCOL = "urn:oasis:names:tc:SAML:2.0:protocol";
	private static String NS_ASSERTION = "urn:oasis:names:tc:SAML:2.0:assertion";
	private static String NS_DSIG = "http://www.w3.org/2000/09/xmldsig#";

	public static String SIGN_SCOPE_Response ="SSR";
	public static String SIGN_SCOPE_ASSERTION ="SSA";

	private void loginProcedureBySAML(Properties ctx, Session session,
			HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, MSAMLConfig m_SAMLConfig ) throws Exception
	{

		String request = httpServletRequest.getParameter("SAMLResponse") ;
		String 	decoded = null;
		try
		{
			decoded = new String(Base64.getDecoder().decode(request));
		} catch (Exception e) {

			//for line feed;
			request = request.replaceAll("\r", "").replaceAll("\n", "");
			try {
				decoded = new String(Base64.getDecoder().decode(request));
			}catch (Exception e2) {

				if(m_SAMLConfig.isVerboseJP())
				{
					String x_Forward_IP = Executions.getCurrent().getHeader("X-Forwarded-For");
	                if (x_Forward_IP == null)
	                	 x_Forward_IP = Executions.getCurrent().getRemoteAddr();

					logAuthFailure.log(x_Forward_IP, "/webui/" + m_SAMLConfig.getJP_SAML_ServletFile() + "?" + MSAMLConfig.JP_SAML_Parameter + "=" + MSAMLConfig.COLUMNNAME_JP_SAML_Parameter_Value
							, "SAML", request);
				}
				throw e2;
			}
		}


        InputStream bais =null;
        try {
			bais = new ByteArrayInputStream(decoded.getBytes("utf-8"));
		} catch (UnsupportedEncodingException e) {
			throw e;
		} catch (Exception e) {
			throw e;
		}

        Document xmlDocument = null;
        try {
        	DocumentBuilderFactory documentbuilderfactory = DocumentBuilderFactory.newInstance();
        	documentbuilderfactory.setIgnoringComments(true);
        	documentbuilderfactory.setNamespaceAware(true);
        	documentbuilderfactory.setXIncludeAware(true);
			xmlDocument = documentbuilderfactory.newDocumentBuilder().parse(bais);
		} catch (SAXException e) {
			throw e;
		} catch (IOException e) {
			throw e;
		} catch (ParserConfigurationException e) {
			throw e;
		} catch (Exception e) {
			throw e;
		}

        Node rootNode = xmlDocument.getFirstChild();
        Node assetionNode = xmlDocument.getElementsByTagNameNS(NS_ASSERTION, "Assertion").item(0);
        Node tempNode = null;
        NodeList tempNodeList = null;
        NamedNodeMap  tempAttributes = null;

        //Check XML Sign
        tempNodeList = xmlDocument.getElementsByTagNameNS(NS_DSIG, "Signature");
        for(int i =0 ; i < tempNodeList.getLength(); i++)
        {
        	tempNode = tempNodeList.item(i);
        	if(tempNode.getParentNode().isEqualNode(rootNode))//Check Response Sign
        	{
        		if(!signatureValidate(tempNode, xmlDocument.getDocumentElement(), m_SAMLConfig, SIGN_SCOPE_Response))
        		{
        			throw new Exception(Msg.getMsg(ctx, "JP_XMLSignature_ValidationError") + " - SAML Response");//Validation Error of XML Signature
        		}

        	}else if(tempNode.getParentNode().isEqualNode(assetionNode)) { //Check Assertion Sign

            	if(!signatureValidate(tempNode, (Element)assetionNode, m_SAMLConfig, SIGN_SCOPE_ASSERTION ))
            	{
            		throw new Exception(Msg.getMsg(ctx, "JP_XMLSignature_ValidationError") + " - Assertion");//Validation Error of XML Signature
            	}
        	}
        }

        //Check Status
        tempNode = xmlDocument.getElementsByTagNameNS(NS_PROTOCOL, "StatusCode").item(0);
        if(tempNode == null)
        {
        	throw new Exception(Msg.getMsg(ctx, "NotFound") + System.lineSeparator() + "StatusCode");

        }else {

        	tempAttributes = tempNode.getAttributes();
        	tempNode = tempAttributes.getNamedItem("Value");
        	String statusCode = tempNode.getNodeValue();
        	if(!statusCode.contains("Success"))
        	{
        		throw new Exception(Msg.getMsg(ctx, "Invalid") + "StatusCode");
        	}

        }

        //Check Available Period
        tempAttributes = rootNode.getAttributes();
        tempNode = tempAttributes.getNamedItem("IssueInstant");
        if(tempNode != null)
        {
	        String timeString = tempNode.getNodeValue();
	        ZonedDateTime zonedDateTimeResponse = ZonedDateTime.parse(timeString);
	        if(!checkAvailablePeriod(zonedDateTimeResponse, m_SAMLConfig.getJP_SAML_AvailablePeriod()))
	        {
	        	//Available Period is expired
	        	throw new Exception(Msg.getMsg(ctx, "JP_AvailablePeriodExpired")
	        							+ System.lineSeparator() + "SAML Response"
	        							+ System.lineSeparator() + "IssueInstant" + " : " + timeString);
	        }

        }

        tempAttributes = assetionNode.getAttributes();
        tempNode = tempAttributes.getNamedItem("IssueInstant");
        if(tempNode != null)
        {
        	String timeString = tempNode.getNodeValue();
	        ZonedDateTime zonedDateTimeAssertion = ZonedDateTime.parse(timeString);
	        if(!checkAvailablePeriod(zonedDateTimeAssertion, m_SAMLConfig.getJP_SAML_AvailablePeriod()))
	        {
	        	//Available Period is expired
	        	throw new Exception(Msg.getMsg(ctx, "JP_AvailablePeriodExpired")
	        							+ System.lineSeparator() + "SAML Assertion"
	        							+ System.lineSeparator() + "IssueInstant" + " : " + timeString);
	        }
        }


        tempNode = xmlDocument.getElementsByTagNameNS(NS_ASSERTION, "Conditions").item(0);
        tempAttributes = tempNode.getAttributes();
        tempNode = tempAttributes.getNamedItem("NotBefore");
        if(tempNode != null)
        {
        	String timeFrom = tempNode.getNodeValue();
	        ZonedDateTime zonedDateTimeConditionsFrom = ZonedDateTime.parse(timeFrom);

	        tempNode = tempAttributes.getNamedItem("NotOnOrAfter");
	        if(tempNode != null)
	        {
	        	String timeTo = tempNode.getNodeValue();
		        ZonedDateTime zonedDateTimeConditionsTo = ZonedDateTime.parse(timeTo);
		        if(!checkAvailablePeriod(zonedDateTimeConditionsFrom, zonedDateTimeConditionsTo))
		        {
		        	throw new Exception(Msg.getMsg(ctx, "JP_AvailablePeriodExpired")
		        								+ System.lineSeparator() + "Conditions"
		        								+ System.lineSeparator() + "NotBefore" + " : " + timeFrom
		        								+ System.lineSeparator() + "NotOnOrAfter" + " : " + timeTo);
		        }

	        }else {

	        	if(!checkAvailablePeriod(zonedDateTimeConditionsFrom, zonedDateTimeConditionsFrom.plusMinutes(m_SAMLConfig.getJP_SAML_AvailablePeriod())))
	        	{
		        	throw new Exception(Msg.getMsg(ctx, "JP_AvailablePeriodExpired")
		        											+ System.lineSeparator() + "Conditions"
										        			+ System.lineSeparator() + "NotBefore" + " : " + timeFrom
															+ System.lineSeparator() + "NotOnOrAfter" + " : null ");
	        	}

	        }
        }


        //Issuer of Assertion
        if(!Util.isEmpty(m_SAMLConfig.getJP_SAML_Assertion_Issuer()))
		{
        	tempNode = xmlDocument.getElementsByTagNameNS(NS_ASSERTION, "Issuer").item(0);
        	String issuer = tempNode.getTextContent();
        	if(!m_SAMLConfig.getJP_SAML_Assertion_Issuer().equals(issuer))
        	{
        		throw new Exception(Msg.getMsg(ctx, "Invalid") + "Issuer - " + issuer);
        	}
  		}


        //Check NameID
        tempNode = xmlDocument.getElementsByTagNameNS(NS_ASSERTION, "NameID").item(0);
        if(tempNode == null)
        {
        	throw new Exception(Msg.getMsg(ctx, "NotFound") + System.lineSeparator()
        							+ Msg.getElement(ctx, "AD_Element_ID") + " : " + "NameID") ;
        }

        String nameID = tempNode.getTextContent();
        if(Util.isEmpty(nameID))
        {
        	throw new Exception(Msg.getMsg(ctx, "JP_Null") + System.lineSeparator()
        							+ Msg.getElement(ctx, "AD_Element_ID") + " : " + "NameID") ;
        }

        Properties properties = new Properties();

        //Set Client Context for Search User
        MClient m_Client = MClient.get(ctx, m_SAMLConfig.getAD_Client_ID());
        Env.setContext(ctx, "#AD_Client_ID", m_Client.getAD_Client_ID());
        properties.setProperty("#AD_Client_ID", String.valueOf(m_Client.getAD_Client_ID()));
        Env.setContext(ctx, "#AD_Client_Name", m_Client.getName());
        properties.setProperty("#AD_Client_Name", m_Client.getName());

        MUser user = MUser.get (Env.getCtx(), nameID);
        if(user == null)
        {
        	throw new Exception(Msg.getMsg(ctx, "NotFound") + System.lineSeparator()
        							+ Msg.getElement(ctx, "AD_User_ID") + " : " + nameID) ;
        }

        Env.setContext(ctx, "#AD_User_ID", user.getAD_User_ID());
        properties.setProperty("#AD_User_ID", String.valueOf(user.getAD_User_ID()));
        Env.setContext(ctx, "#AD_User_Name", user.getName());
        properties.setProperty("#AD_User_Name", user.getName());
        Env.setContext(ctx, "#SalesRep_ID", user.getAD_User_ID());
        properties.setProperty("#SalesRep_ID", String.valueOf(user.getAD_User_ID()));

        loadUserPreference(user.getAD_User_ID());

        //Lang & Client
        String langName = userPreference.getProperty(UserPreference.P_LANGUAGE);
        String p_Client = userPreference.getProperty(UserPreference.P_CLIENT);

        if(Util.isEmpty(langName) || Util.isEmpty(p_Client))//if p_Client is empty, it is first login.
            langName = m_Client.getAD_Language();//ja_JP,en_US

        Language tmp = Language.getLanguage(langName);
        Language language = new Language(tmp.getName(), tmp.getAD_Language(), tmp.getLocale(), tmp.isDecimalPoint(),
                									tmp.getDateFormat().toPattern(), tmp.getMediaSize());
        Env.verifyLanguage(ctx, language);

        Env.setContext(ctx, "#AD_Language", language.getAD_Language());
        properties.setProperty("#AD_Language", language.getAD_Language());
        Env.setContext(ctx, "#Locale", language.getLocale().toString());
        properties.setProperty("#Locale", language.getLocale().toString());
        Env.setContext(ctx, "#LanguageName", language.getName());
        properties.setProperty("#LanguageName", language.getName());

        langSession = language.getAD_Language();

        if(Util.isEmpty(p_Client))//client
        {
        	;

        }else {

	        m_Client = MClient.get(ctx, Integer.parseInt(p_Client));
	        if(m_SAMLConfig.getAD_Client_ID() != m_Client.getAD_Client_ID())
	        {
	        	Env.setContext(ctx, "#AD_Client_ID", m_Client.getAD_Client_ID());
	        	properties.setProperty("#AD_Client_ID", String.valueOf(m_Client.getAD_Client_ID()));
	        }
        }

        boolean isShowRolePanel = false;

        //Role
        String p_Role = userPreference.getProperty(UserPreference.P_ROLE);
        MRole m_Role = null;
        if(Util.isEmpty(p_Role))
        {
        	isShowRolePanel = true;
    		String sql = "SELECT r.AD_Role_ID FROM AD_User_Roles ur "
    							+ " INNER JOIN AD_Role r ON (ur.AD_Role_ID = r.AD_Role_ID) "
    							+ " WHERE ur.AD_Client_ID =? AND ur.AD_User_ID =? "
    							+ " ORDER BY r.Name";

    		PreparedStatement pstmt = null;
    		ResultSet rs = null;
    		try
    		{
    			pstmt = DB.prepareStatement(sql.toString(), null);
    			pstmt.setInt(1, m_SAMLConfig.getAD_Client_ID());
    			pstmt.setInt(2, user.getAD_User_ID());
    			rs = pstmt.executeQuery();

    			if (!rs.next())
    			{
    				throw new Exception(Msg.getMsg(ctx, "NotFound") + System.lineSeparator() + Msg.getElement(ctx, "AD_Role_ID"));
    			}else {

    				m_Role =  MRole.get(ctx, rs.getInt(1));
    			}


    		}catch (SQLException ex){
    			throw ex;
    		}catch (Exception ex){
    			throw ex;
    		}finally{
    			DB.close(rs, pstmt);
    			rs = null; pstmt = null;
    		}

        	if(m_Role == null)
        	{
        		throw new Exception(Msg.getMsg(ctx, "NotFound") + System.lineSeparator() + Msg.getElement(ctx, "AD_Role_ID"));
        	}

        }else {

        	m_Role = MRole.get(ctx, Integer.parseInt(p_Role), user.getAD_User_ID(), false);
        	MUserRoles[] userRoles = MUserRoles.getOfUser(ctx, user.getAD_User_ID());
        	boolean isOK = false;
        	for(int i = 0; i < userRoles.length; i++)
        	{
        		if(m_Role.getAD_Role_ID() == userRoles[i].getAD_Role_ID())
        		{
        			if(userRoles[i].isActive())
        			{
        				isOK = true;
        				break;
        			}
        		}
        	}

        	if(!isOK)
        	{
        		throw new Exception(Msg.getMsg(ctx, "NotFound") + System.lineSeparator() + Msg.getElement(ctx, "AD_Role_ID"));
        	}

        }

        Env.setContext(ctx, "#AD_Role_ID", m_Role.getAD_Role_ID());
        properties.setProperty("#AD_Role_ID", String.valueOf(m_Role.getAD_Role_ID()));
        Env.setContext(ctx, "#AD_Role_Name", m_Role.getName());
        properties.setProperty("#AD_Role_Name", m_Role.getName());


        //Org
        String p_Org = userPreference.getProperty(UserPreference.P_ORG);
        MOrg m_Org = null;
        if(Util.isEmpty(p_Org))
        {
        	isShowRolePanel = true;

        }else {

        	String orgWhere = m_Role.getOrgWhere(false);
        	if(Util.isEmpty(orgWhere) || p_Org.equals("0"))
        	{
        		;//Noting to do

        	}else {

        		if(!orgWhere.contains(p_Org))
        		{
        			throw new Exception(Msg.getMsg(ctx, "NotFound") + System.lineSeparator() + Msg.getElement(ctx, "AD_Org_ID"));
        		}

        	}

        	m_Org = MOrg.get(ctx, Integer.parseInt(p_Org));

        	Env.setContext(ctx, "#AD_Org_ID", m_Org.getAD_Org_ID());
        	properties.setProperty("#AD_Org_ID", String.valueOf(m_Org.getAD_Org_ID()));

        	if(m_Org.getAD_Org_ID()==0)
        	{
                Env.setContext(ctx, "#AD_Org_Name", "*");
     	        properties.setProperty("#AD_Org_Name","*");

        	}else {

                Env.setContext(ctx, "#AD_Org_Name", m_Org.getName());
     	        properties.setProperty("#AD_Org_Name", m_Org.getName());

        	}

        }


    	String p_Warehouse = userPreference.getProperty(UserPreference.P_WAREHOUSE);
    	MWarehouse m_Warehouse = null;
    	if(Util.isEmpty(p_Warehouse))
    	{
    		if(m_Client.getAD_Client_ID() == 0)
    		{
    			;//Noting to do

    		}else if(m_Org == null) {

    			isShowRolePanel = true;

    		}else if(m_Org.getAD_Org_ID() == 0){

    			;//Noting to do

    		}else {

    			isShowRolePanel = true;
    		}

    	}else if(m_Org != null ){

    		m_Warehouse = MWarehouse.get(ctx, Integer.parseInt(p_Warehouse));
    		if(m_Org.getAD_Org_ID() == m_Warehouse.getAD_Org_ID())
    		{
    			Env.setContext(ctx, "#M_Warehouse_ID", m_Warehouse.getM_Warehouse_ID());
        		properties.setProperty("#M_Warehouse_ID", String.valueOf(m_Warehouse.getM_Warehouse_ID()));

    		}else {

    			isShowRolePanel = true;
    		}

    	}


        if (isShowRolePanel)
        {
        	loginDesktop = new WLogin(this);
        	loginDesktop.createPart(this.getPage());

	        ServerContext.setCurrentInstance(properties);

	        loginDesktop.changeRole(language.getLocale(), properties);
	        loginDesktop.getComponent().getRoot().addEventListener(Events.ON_CLIENT_INFO, this);

        }else {

        	loadOtherPreferences(ctx, properties, m_Client, m_Org);

    		String error = ModelValidationEngine.get().loginComplete(m_Client.getAD_Client_ID(), m_Org.getAD_Org_ID(), m_Role.getAD_Role_ID(), user.getAD_User_ID());
    		if (error != null && error.length() > 0)
    		{
    			throw new Exception("Refused: " + error);
    		}

        	loginCompleted();

        }

        Executions.getCurrent().getDesktop().enableServerPush(true);

        Executions.getCurrent().getDesktop().addListener(new DrillCommand());
        Executions.getCurrent().getDesktop().addListener(new TokenCommand());
        Executions.getCurrent().getDesktop().addListener(new ZoomCommand());

        eventThreadEnabled = Executions.getCurrent().getDesktop().getWebApp().getConfiguration().isEventThreadEnabled();

        return ;
	}

	private boolean signatureValidate(Node dSignNode, Element targetIDElement, MSAMLConfig m_SAMLConfig, String ss ) throws Exception
	{
		JPiereSAMLKeySelector keySelector = new JPiereSAMLKeySelector();
		keySelector.setMSAMLConfig(m_SAMLConfig);
		keySelector.setSignScope(ss);
        DOMValidateContext valContext = new DOMValidateContext(keySelector, dSignNode);
        valContext.setIdAttributeNS(targetIDElement, null, "ID");

        XMLSignatureFactory factory = XMLSignatureFactory.getInstance("DOM");
        XMLSignature signature = factory.unmarshalXMLSignature(valContext);

		 boolean coreValidity = signature.validate(valContext);

        // Check core validation status
//        if (!coreValidity)
//        {
//            System.err.println("Signature failed core validation");
//            boolean sv = signature.getSignatureValue().validate(valContext);
//
//            System.out.println("signature validation status: " + sv);
//            // check the validation status of each Reference
//            Iterator i = signature.getSignedInfo().getReferences().iterator();
//            for (int j=0; i.hasNext(); j++)
//            {
//                boolean refValid = ((Reference) i.next()).validate(valContext);
//                System.out.println("ref["+j+"] validity status: " + refValid);
//            }
//
//        } else {
//            System.out.println("Signature passed core validation");
//        }

		return coreValidity;
	}

	private boolean checkAvailablePeriod(ZonedDateTime checkZonedDateTime, int availableMin)
	{
		if(checkZonedDateTime == null ||  availableMin == 0)
			return true;

		ZoneId  zoneID = checkZonedDateTime.getZone();
	    ZonedDateTime zonedDateTimeNow = ZonedDateTime.now(zoneID);

		return checkAvailablePeriod(checkZonedDateTime, zonedDateTimeNow.minusMinutes(availableMin), zonedDateTimeNow.plusMinutes(availableMin));
	}

	private boolean checkAvailablePeriod(ZonedDateTime zonedAvailableTimeFrom, ZonedDateTime zonedAvailableTimeTo)
	{
		if(zonedAvailableTimeFrom == null || zonedAvailableTimeTo == null)
			return true;

		ZoneId  zoneID = zonedAvailableTimeFrom.getZone();
		ZonedDateTime zonedDateTimeNow = ZonedDateTime.now(zoneID);

		return checkAvailablePeriod(zonedDateTimeNow, zonedAvailableTimeFrom, zonedAvailableTimeTo);
	}

	private boolean checkAvailablePeriod(ZonedDateTime checkZonedDateTime, ZonedDateTime zonedAvailableTimeFrom, ZonedDateTime zonedAvailableTimeTo)
	{

		if(checkZonedDateTime == null || zonedAvailableTimeFrom == null || zonedAvailableTimeTo == null)
			return true;


	    if( checkZonedDateTime.isAfter(zonedAvailableTimeFrom)
	    	&& checkZonedDateTime.isBefore(zonedAvailableTimeTo))
	    {
	    	return true;

	    }else {

	    	return false;
	    }

	}

	private void loadOtherPreferences(Properties ctx , Properties properties, MClient m_Client, MOrg m_Org)
	{
		//	Date (default today)
		long today = System.currentTimeMillis();
		Env.setContext(ctx, "#Date", new java.sql.Timestamp(today));
		properties.setProperty("#Date", String.valueOf(new java.sql.Timestamp(today)));

		//	Other Settings
		Env.setContext(ctx, "#YYYY", "Y");
		properties.setProperty("#YYYY", "Y");
		Env.setContext(ctx, "#StdPrecision", 2);
		properties.setProperty("#StdPrecision", String.valueOf(2));

		//AccountSchema Info (first)
		String sql = "SELECT * "
				+ "FROM C_AcctSchema a, AD_ClientInfo c "
				+ "WHERE a.C_AcctSchema_ID=c.C_AcctSchema1_ID "
				+ "AND c.AD_Client_ID=?";
			PreparedStatement pstmt = null;
			ResultSet rs = null;
			try
			{

				int C_AcctSchema_ID = 0;

				pstmt = DB.prepareStatement(sql, null);
				pstmt.setInt(1, m_Client.getAD_Client_ID());
				rs = pstmt.executeQuery();

				if (!rs.next())
				{
					return;
				}
				else
				{
					//	Accounting Info
					C_AcctSchema_ID = rs.getInt("C_AcctSchema_ID");
					Env.setContext(ctx, "$C_AcctSchema_ID", C_AcctSchema_ID);
					properties.setProperty("$C_AcctSchema_ID", String.valueOf(C_AcctSchema_ID));
					Env.setContext(ctx, "$C_Currency_ID", rs.getInt("C_Currency_ID"));
					properties.setProperty("$C_Currency_ID", String.valueOf(rs.getInt("C_Currency_ID")));
					Env.setContext(ctx, "$HasAlias", rs.getString("HasAlias"));
					properties.setProperty("$HasAlias", String.valueOf(rs.getString("HasAlias")));
				}
				DB.close(rs, pstmt);
				rs = null; pstmt = null;

				/**Define AcctSchema , Currency, HasAlias for Multi AcctSchema**/
				MAcctSchema[] ass = MAcctSchema.getClientAcctSchema(Env.getCtx(), m_Client.getAD_Client_ID());
				if(ass != null && ass.length > 1)
				{
					for(MAcctSchema as : ass)
					{
						C_AcctSchema_ID  = MClientInfo.get(Env.getCtx(), m_Client.getAD_Client_ID()).getC_AcctSchema1_ID();
						if (as.getAD_OrgOnly_ID() != 0)
						{
							if (as.isSkipOrg(m_Org.getAD_Org_ID()))
							{
								continue;
							}
							else
							{
								C_AcctSchema_ID = as.getC_AcctSchema_ID();
								Env.setContext(ctx, "$C_AcctSchema_ID", C_AcctSchema_ID);
								properties.setProperty("$C_AcctSchema_ID", String.valueOf(C_AcctSchema_ID));
								Env.setContext(ctx, "$C_Currency_ID", as.getC_Currency_ID());
								properties.setProperty("$C_Currency_ID", String.valueOf(as.getC_Currency_ID()));
								Env.setContext(ctx, "$HasAlias", as.isHasAlias());
								properties.setProperty("$HasAlias", String.valueOf(as.isHasAlias()));
								break;
							}
						}
					}
				}

				//	Accounting Elements
				sql = "SELECT ElementType "
					+ "FROM C_AcctSchema_Element "
					+ "WHERE C_AcctSchema_ID=?"
					+ " AND IsActive='Y'";
				pstmt = DB.prepareStatement(sql, null);
				pstmt.setInt(1, C_AcctSchema_ID);
				rs = pstmt.executeQuery();
				while (rs.next())
				{
					Env.setContext(ctx, "$Element_" + rs.getString("ElementType"), "Y");
					properties.setProperty("$Element_" + rs.getString("ElementType"), "Y");
				}
				DB.close(rs, pstmt);
				rs = null; pstmt = null;

//				This reads all relevant window neutral defaults
				//	overwriting superseeded ones.  Window specific is read in Mainain
				sql = "SELECT Attribute, Value, AD_Window_ID, AD_Process_ID, AD_InfoWindow_ID, PreferenceFor "
					+ "FROM AD_Preference "
					+ "WHERE AD_Client_ID IN (0, @#AD_Client_ID@)"
					+ " AND AD_Org_ID IN (0, @#AD_Org_ID@)"
					+ " AND (AD_User_ID IS NULL OR AD_User_ID=0 OR AD_User_ID=@#AD_User_ID@)"
					+ " AND IsActive='Y' "
					+ "ORDER BY Attribute, AD_Client_ID, AD_User_ID DESC, AD_Org_ID";
					//	the last one overwrites - System - Client - User - Org - Window
				sql = Env.parseContext(ctx, 0, sql, false);
				if (sql.length() == 0)
				{
//					log.log(Level.SEVERE, "loadPreferences - Missing Environment");
					;
				}
				else
				{
					pstmt = DB.prepareStatement(sql, null);
					rs = pstmt.executeQuery();
					while (rs.next())
					{
						int AD_Window_ID = rs.getInt(3);
						boolean isAllWindow = rs.wasNull();
						int AD_Process_ID = rs.getInt(4);
						int AD_InfoWindow_ID = rs.getInt(5);
						String PreferenceFor = rs.getString(6);
						String at = "";

						// preference for window
						if ("W".equals(PreferenceFor)){
						  if (isAllWindow)
							at = "P|" + rs.getString(1);
						  else
							at = "P" + AD_Window_ID + "|" + rs.getString(1);
						}else if ("P".equals(PreferenceFor)){ // preference for processs
							// when apply for all window or all process format is "P0|0|m_Attribute;
							at = "P" + AD_Window_ID + "|" + AD_InfoWindow_ID + "|" + AD_Process_ID + "|" + rs.getString(1);
						}else if ("I".equals(PreferenceFor)){ // preference for infoWindow
							at = "P" + AD_Window_ID + "|" + AD_InfoWindow_ID + "|" + rs.getString(1);
						}

						String va = rs.getString(2);
						Env.setContext(ctx, at, va);
						properties.setProperty(at, va);
					}
					DB.close(rs, pstmt);
					rs = null; pstmt = null;
				}

				//	Default Values
//				if (log.isLoggable(Level.INFO)) log.info("Default Values ...");
				sql = "SELECT t.TableName, c.ColumnName "
					+ "FROM AD_Column c "
					+ " INNER JOIN AD_Table t ON (c.AD_Table_ID=t.AD_Table_ID) "
					+ "WHERE c.IsKey='Y' AND t.IsActive='Y' AND t.IsView='N'"
					+ " AND EXISTS (SELECT * FROM AD_Column cc "
					+ " WHERE ColumnName = 'IsDefault' AND t.AD_Table_ID=cc.AD_Table_ID AND cc.IsActive='Y')";
				pstmt = DB.prepareStatement(sql, null);
				rs = pstmt.executeQuery();
				while (rs.next())
					loadDefault (rs.getString(1), rs.getString(2), ctx , properties);
			}
			catch (SQLException e)
			{
//				log.log(Level.SEVERE, "loadPreferences", e);
			}
			finally
			{
				DB.close(rs, pstmt);
				rs = null; pstmt = null;
			}
			//	Country
			Env.setContext(ctx, "#C_Country_ID", MCountry.getDefault(ctx).getC_Country_ID());
			properties.setProperty("#C_Country_ID", String.valueOf(MCountry.getDefault(ctx).getC_Country_ID()) );

			// Call ModelValidators afterLoadPreferences - teo_sarca FR [ 1670025 ]
			ModelValidationEngine.get().afterLoadPreferences(ctx);
			return ;

	}

	/**
	 *	Load Default Value for Table into Context.
	 *  @param TableName table name
	 *  @param ColumnName column name
	 */
	private void loadDefault (String TableName, String ColumnName, Properties ctx , Properties properties)
	{
		if (TableName.startsWith("AD_Window")
			|| TableName.startsWith("AD_PrintFormat")
			|| TableName.startsWith("AD_Workflow")
			|| TableName.startsWith("M_Locator") )
			return;
		String value = null;
		//
		String sql = "SELECT " + ColumnName + " FROM " + TableName	//	most specific first
			+ " WHERE IsDefault='Y' AND IsActive='Y' ORDER BY AD_Client_ID DESC, AD_Org_ID DESC";
		sql = MRole.getDefault(ctx, false).addAccessSQL(sql,
			TableName, MRole.SQL_NOTQUALIFIED, MRole.SQL_RO);
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try
		{
			pstmt = DB.prepareStatement(sql, null);
			rs = pstmt.executeQuery();
			if (rs.next())
				value = rs.getString(1);
		}
		catch (SQLException e)
		{
//			log.log(Level.SEVERE, TableName + " (" + sql + ")", e);
			return;
		}
		finally
		{
			DB.close(rs, pstmt);
			rs = null; pstmt = null;
		}
		//	Set Context Value
		if (value != null && value.length() != 0)
		{
			if (TableName.equals("C_DocType"))
			{
				Env.setContext(ctx, "#C_DocTypeTarget_ID", value);
				properties.setProperty("#C_DocTypeTarget_ID", value);
			}else {
				Env.setContext(ctx, "#" + ColumnName, value);
				properties.setProperty("#" + ColumnName, value);
			}
		}
	}	//	loadDefault
}
